@@ -3,13 +3,14 @@ import numpy as np
 import os
 from collections import OrderedDict
 
-from madminer.utils.interfaces.madminer_hdf5 import (
-    save_events_to_madminer_file,
-    load_madminer_settings,
-    save_nuisance_setup_to_madminer_file,
-)
+from madminer.models import Cut
+from madminer.models import Observable
+from madminer.models import NuisanceParameter
 from madminer.utils.interfaces.delphes import run_delphes
 from madminer.utils.interfaces.delphes_root import parse_delphes_root_file
+from madminer.utils.interfaces.hdf5 import load_madminer_settings
+from madminer.utils.interfaces.hdf5 import save_events
+from madminer.utils.interfaces.hdf5 import save_nuisance_setup
 from madminer.utils.interfaces.hepmc import extract_weight_order
 from madminer.utils.interfaces.lhe import parse_lhe_file, extract_nuisance_parameters_from_lhe_file
 from madminer.sampling import combine_and_shuffle
@@ -62,12 +63,9 @@ class DelphesReader:
 
         # Initialize observables
         self.observables = OrderedDict()
-        self.observables_required = OrderedDict()
-        self.observables_defaults = OrderedDict()
 
         # Initialize cuts
         self.cuts = []
-        self.cuts_default_pass = []
 
         # Initialize acceptance cuts
         self.acceptance_pt_min_e = None
@@ -83,7 +81,7 @@ class DelphesReader:
         self.reference_benchmark = None
         self.observations = None
         self.weights = None
-        self.events_sampling_benchmark_ids = None
+        self.events_sampling_benchmark_ids = []
 
         # Initialize event summary
         self.signal_events_per_benchmark = None
@@ -91,37 +89,30 @@ class DelphesReader:
 
         # Information from .h5 file
         self.filename = filename
-        (parameters, benchmarks, _, _, _, _, _, self.systematics, _, _, _, _, _, _) = load_madminer_settings(
-            filename, include_nuisance_benchmarks=False
-        )
+
+        (
+            _,
+            benchmarks,
+            _,
+            _,
+            _,
+            _,
+            _,
+            self.systematics,
+            _,
+            _,
+            _,
+            _,
+            _,
+            _,
+        ) = load_madminer_settings(filename, include_nuisance_benchmarks=False)
+
         self.benchmark_names_phys = list(benchmarks.keys())
         self.n_benchmarks_phys = len(benchmarks)
 
         # Initialize nuisance parameters
         self.nuisance_parameters = OrderedDict()
 
-    @staticmethod
-    def _check_python_syntax(expression):
-        """
-        Evaluates a Python expression to check for syntax errors
-
-        Parameters
-        ----------
-        expression : str
-            Python expression to be evaluated. The evaluation raises either SyntaxError or NameError
-
-        Returns
-        -------
-            None
-        """
-
-        try:
-            eval(expression)
-        except SyntaxError:
-            raise ValueError("The provided Python expression is invalid")
-        except NameError:
-            pass
-
     @staticmethod
     def _check_sample_elements(this_elements, n_events=None):
         """ Sanity checks """
@@ -378,19 +369,19 @@ class DelphesReader:
         Returns
         -------
             None
-
         """
 
-        self._check_python_syntax(definition)
-
         if required:
             logger.debug("Adding required observable %s = %s", name, definition)
         else:
             logger.debug("Adding optional observable %s = %s with default %s", name, definition, default)
 
-        self.observables[name] = definition
-        self.observables_required[name] = required
-        self.observables_defaults[name] = default
+        self.observables[name] = Observable(
+            name=name,
+            val_expression=definition,
+            val_default=default,
+            is_required=required,
+        )
 
     def add_observable_from_function(self, name, fn, required=False, default=None):
         """
@@ -419,7 +410,6 @@ class DelphesReader:
         Returns
         -------
             None
-
         """
 
         if required:
@@ -429,9 +419,12 @@ class DelphesReader:
                 "Adding optional observable %s defined through external function with default %s", name, default
             )
 
-        self.observables[name] = fn
-        self.observables_required[name] = required
-        self.observables_defaults[name] = default
+        self.observables[name] = Observable(
+            name=name,
+            val_expression=fn,
+            val_default=default,
+            is_required=required,
+        )
 
     def add_default_observables(
         self,
@@ -473,8 +466,8 @@ class DelphesReader:
         Returns
         -------
             None
-
         """
+
         logger.debug("Adding default observables")
 
         # ETMiss
@@ -515,8 +508,7 @@ class DelphesReader:
                         default=0.0,
                     )
 
-    def add_cut(self, definition, pass_if_not_parsed=False):
-
+    def add_cut(self, definition, required=False):
         """
         Adds a cut as a string that can be parsed by Python's `eval()` function and returns a bool.
 
@@ -533,38 +525,33 @@ class DelphesReader:
             PDG particle ID. For instance, `"len(e) >= 2"` requires at least two electrons passing the acceptance cuts,
             while `"mu[0].charge > 0."` specifies that the hardest muon is positively charged.
 
-        pass_if_not_parsed : bool, optional
+        required : bool, optional
             Whether the cut is passed if the observable cannot be parsed. Default value: False.
 
         Returns
         -------
             None
-
         """
 
-        self._check_python_syntax(definition)
-
         logger.debug("Adding cut %s", definition)
 
-        self.cuts.append(definition)
-        self.cuts_default_pass.append(pass_if_not_parsed)
+        self.cuts.append(Cut(
+            name="CUT",
+            val_expression=definition,
+            is_required=required,
+        ))
 
     def reset_observables(self):
         """ Resets all observables. """
 
         logger.debug("Resetting observables")
-
         self.observables = OrderedDict()
-        self.observables_required = OrderedDict()
-        self.observables_defaults = OrderedDict()
 
     def reset_cuts(self):
         """ Resets all cuts. """
 
         logger.debug("Resetting cuts")
-
         self.cuts = []
-        self.cuts_default_pass = []
 
     def analyse_delphes_samples(
         self,
@@ -612,7 +599,7 @@ class DelphesReader:
         self.observations = None
         self.weights = None
         self.nuisance_parameters = OrderedDict()
-        self.events_sampling_benchmark_ids = None
+        self.events_sampling_benchmark_ids = []
         self.signal_events_per_benchmark = [0 for _ in range(self.n_benchmarks_phys)]
         self.background_events = 0
 
@@ -670,7 +657,7 @@ class DelphesReader:
             else:
                 idx = self.benchmark_names_phys.index(sampling_benchmark)
                 self.signal_events_per_benchmark[idx] += this_n_events
-            this_events_sampling_benchmark_ids = np.array([idx] * this_n_events, dtype=np.int)
+            this_events_sampling_benchmark_ids = np.array([idx] * this_n_events, dtype=int)
 
             # First results
             if self.observations is None and self.weights is None:
@@ -761,27 +748,34 @@ class DelphesReader:
 
         # Store nuisance parameters
         for systematics_name, nuisance_info in systematics_dict.items():
-            for nuisance_parameter_name, ((benchmark0, weight0), (benchmark1, weight1), _) in nuisance_info.items():
+            for nuisance_param_name, ((benchmark0, weight0), (benchmark1, weight1), _) in nuisance_info.items():
+                nuisance_param = self.nuisance_parameters.get(nuisance_param_name)
+
+                if nuisance_param is None:
+                    raise RuntimeError(f"Nuisance parameter {nuisance_param_name} does not exist")
                 if (
-                    self.nuisance_parameters is not None
-                    and nuisance_parameter_name in self.nuisance_parameters
-                    and (systematics_name, benchmark0, benchmark1) != self.nuisance_parameters[nuisance_parameter_name]
+                    nuisance_param.systematic != systematics_name
+                    or nuisance_param.benchmark_pos != benchmark0
+                    or nuisance_param.benchmark_neg != benchmark1
                 ):
                     raise RuntimeError(
-                        f"Inconsistent information for same nuisance parameter {nuisance_parameter_name}. "
-                        f"Old: {self.nuisance_parameters[nuisance_parameter_name]}. "
+                        f"Inconsistent information for same nuisance parameter {nuisance_param_name}. "
+                        f"Old: {nuisance_param}. "
                         f"New: {(systematics_name, benchmark0, benchmark1)}."
                     )
-                self.nuisance_parameters[nuisance_parameter_name] = (systematics_name, benchmark0, benchmark1)
+
+                self.nuisance_parameters[nuisance_param_name] = NuisanceParameter(
+                    name=nuisance_param_name,
+                    systematic=systematics_name,
+                    benchmark_pos=benchmark0,
+                    benchmark_neg=benchmark1,
+                )
 
         # Calculate observables and weights in Delphes ROOT file
         this_observations, this_weights, cut_filter = parse_delphes_root_file(
             delphes_file,
             self.observables,
-            self.observables_required,
-            self.observables_defaults,
             self.cuts,
-            self.cuts_default_pass,
             weight_labels,
             use_generator_truth=generator_truth,
             delete_delphes_sample_file=delete_delphes_files,
@@ -886,23 +880,25 @@ class DelphesReader:
         weight_names = list(self.weights.keys())
         logger.debug("Weight names: %s", weight_names)
 
-        save_nuisance_setup_to_madminer_file(
-            filename_out,
-            weight_names,
-            self.nuisance_parameters,
+        save_nuisance_setup(
+            file_name=filename_out,
+            file_override=True,
+            nuisance_benchmarks=weight_names,
+            nuisance_parameters=self.nuisance_parameters,
             reference_benchmark=self.reference_benchmark,
-            copy_from=self.filename,
+            copy_from_path=self.filename,
         )
 
         # Save events
-        save_events_to_madminer_file(
-            filename_out,
-            self.observables,
-            self.observations,
-            self.weights,
-            self.events_sampling_benchmark_ids,
-            self.signal_events_per_benchmark,
-            self.background_events,
+        save_events(
+            file_name=filename_out,
+            file_override=True,
+            observables=self.observables,
+            observations=self.observations,
+            weights=self.weights,
+            sampling_benchmarks=self.events_sampling_benchmark_ids,
+            num_signal_events=self.signal_events_per_benchmark,
+            num_background_events=self.background_events,
         )
 
         if shuffle:
