@@ -1,11 +1,8 @@
-from __future__ import absolute_import, division, print_function
-
-import six
-import logging
 import json
+import logging
 import numpy as np
-
 from madminer.utils.various import create_missing_folders, load_and_check
+
 from .base import Estimator
 from .double_parameterized_ratio import DoubleParameterizedRatioEstimator
 from .likelihood import LikelihoodEstimator
@@ -13,11 +10,6 @@ from .parameterized_ratio import ParameterizedRatioEstimator
 from .score import ScoreEstimator
 
 
-try:
-    FileNotFoundError
-except NameError:
-    FileNotFoundError = IOError
-
 logger = logging.getLogger(__name__)
 
 
@@ -76,7 +68,7 @@ class Ensemble:
                 if isinstance(estimator, Estimator):
                     self.estimators.append(estimator)
                 else:
-                    raise ValueError("Entry {} in estimators is neither str nor Estimator instance")
+                    raise ValueError(f"Estimator {estimator} is not an Estimator instance")
 
         self.n_estimators = len(self.estimators)
         self._check_consistency()
@@ -93,10 +85,10 @@ class Ensemble:
         Returns
         -------
             None
-
         """
+
         if not isinstance(estimator, Estimator):
-            raise ValueError("Entry {} in estimators is neither str nor Estimator instance")
+            raise ValueError(f"Estimator {estimator} is not an Estimator instance")
 
         self.estimators.append(estimator)
         self.n_estimators = len(self.estimators)
@@ -116,11 +108,13 @@ class Ensemble:
 
         Returns
         -------
-            None
-
+        result: ndarray
+            Training and validation losses from estimator training
+            
         """
 
-        self.estimators[i].train(**kwargs)
+        result=self.estimators[i].train(**kwargs)
+        return result
 
     def train_all(self, **kwargs):
         """
@@ -135,24 +129,30 @@ class Ensemble:
 
         Returns
         -------
-            None
+        result_list: list of ndarray
+            List of training and validation losses from estimator training
 
         """
         logger.info("Training %s estimators in ensemble", self.n_estimators)
 
-        for key, value in six.iteritems(kwargs):
+        result_list=[]
+
+        for key, value in kwargs.items():
             if not isinstance(value, list):
                 kwargs[key] = [value for _ in range(self.n_estimators)]
 
-            assert len(kwargs[key]) == self.n_estimators, "Keyword {} has wrong length {}".format(key, len(value))
+            assert len(kwargs[key]) == self.n_estimators, f"Keyword {key} has wrong length {len(value)}"
 
         for i, estimator in enumerate(self.estimators):
             kwargs_this_estimator = {}
-            for key, value in six.iteritems(kwargs):
+            for key, value in kwargs.items():
                 kwargs_this_estimator[key] = value[i]
 
             logger.info("Training estimator %s / %s in ensemble", i + 1, self.n_estimators)
-            estimator.train(**kwargs_this_estimator)
+            result=estimator.train(**kwargs_this_estimator)
+            result_list.append(result)
+
+        return result_list
 
     def evaluate_log_likelihood(self, estimator_weights=None, calculate_covariance=False, **kwargs):
         """
@@ -177,7 +177,6 @@ class Ensemble:
 
         covariance : ndarray or None
             If calculate_covariance is True, the covariance matrix between the estimators. Otherwise None.
-
         """
 
         logger.debug("Evaluating %s estimators in ensemble", self.n_estimators)
@@ -231,7 +230,6 @@ class Ensemble:
 
         covariance : ndarray or None
             If calculate_covariance is True, the covariance matrix between the estimators. Otherwise None.
-
         """
 
         logger.debug("Evaluating %s estimators in ensemble", self.n_estimators)
@@ -285,7 +283,6 @@ class Ensemble:
 
         covariance : ndarray or None
             If calculate_covariance is True, the covariance matrix between the estimators. Otherwise None.
-
         """
 
         logger.debug("Evaluating %s estimators in ensemble", self.n_estimators)
@@ -338,6 +335,7 @@ class Ensemble:
         more precise and is the default.
 
         In the "score" mode, the covariance matrix of the final result is calculated in the following way:
+
         - For each event `x` and each estimator `a`, the "shifted" predicted score is calculated as
           `t_a'(x) = t(x) + 1/sqrt(n) * (t_a(x) - t(x))`. Here `t(x)` is the mean score (averaged over the ensemble)
           for this event, `t_a(x)` is the prediction of estimator `a` for this event, and `n` is the number of
@@ -396,6 +394,7 @@ class Ensemble:
             The covariance of the estimated Fisher information matrix. This object has four indices, `cov_(ij)(i'j')`,
             ordered as i j i' j'. It has shape `(n_parameters, n_parameters, n_parameters, n_parameters)`.
         """
+
         logger.debug("Evaluating Fisher information for %s estimators in ensemble", self.n_estimators)
 
         # Check ensemble
@@ -406,8 +405,8 @@ class Ensemble:
             )
 
         # Check input
-        if mode not in ["score", "information"]:
-            raise ValueError("Unknown mode {}, has to be 'score' or 'information'!".format(mode))
+        if mode not in ["score", "information", "modified_score"]:
+            raise ValueError(f"Unknown mode {mode}!")
 
         # Calculate estimator_weights of each estimator in vote
         if estimator_weights is None:
@@ -448,7 +447,7 @@ class Ensemble:
         # "modified_score" mode:
         elif mode == "modified_score":
             # Load training data
-            if isinstance(x, six.string_types):
+            if isinstance(x, str):
                 x = load_and_check(x)
             n_samples = x.shape[0]
 
@@ -464,7 +463,7 @@ class Ensemble:
             # Get ensemble mean and ensemble covariance
             score_mean = np.mean(score_predictions, axis=0)  # (n_events, n_parameters)
 
-            # For uncertainty calculation: calculate points betweeen mean and original predictions with same mean and
+            # For uncertainty calculation: calculate points between mean and original predictions with same mean and
             # variance / n compared to the original predictions
             score_shifted_predictions = (score_predictions - score_mean[np.newaxis, :, :]) / self.n_estimators ** 0.5
             score_shifted_predictions = score_mean[np.newaxis, :, :] + score_shifted_predictions
@@ -511,7 +510,7 @@ class Ensemble:
         # "score" mode:
         elif mode == "score":
             # Load training data
-            if isinstance(x, six.string_types):
+            if isinstance(x, str):
                 x = load_and_check(x)
             n_samples = x.shape[0]
 
@@ -527,7 +526,7 @@ class Ensemble:
             # Get ensemble mean and ensemble covariance
             score_mean = np.mean(score_predictions, axis=0)  # (n_events, n_parameters)
 
-            # For uncertainty calculation: calculate points betweeen mean and original predictions with same mean and
+            # For uncertainty calculation: calculate points between mean and original predictions with same mean and
             # variance / n compared to the original predictions
             score_shifted_predictions = epsilon_shift * (score_predictions - score_mean[np.newaxis, :, :])
             score_shifted_predictions = score_mean[np.newaxis, :, :] + score_shifted_predictions
@@ -594,7 +593,6 @@ class Ensemble:
         Returns
         -------
             None
-
         """
 
         # Check paths
@@ -604,12 +602,12 @@ class Ensemble:
         logger.debug("Saving ensemble setup to %s/ensemble.json", folder)
         settings = {"estimator_type": self.estimator_type, "n_estimators": self.n_estimators}
 
-        with open(folder + "/ensemble.json", "w") as f:
+        with open(f"{folder}/ensemble.json", "w") as f:
             json.dump(settings, f)
 
         # Save estimators
         for i, estimator in enumerate(self.estimators):
-            estimator.save(folder + "/estimator_" + str(i), save_model=save_model)
+            estimator.save(f"{folder}/estimator_{i}", save_model=save_model)
 
     def load(self, folder):
         """
@@ -623,11 +621,11 @@ class Ensemble:
         Returns
         -------
             None
-
         """
+
         # Load ensemble settings
         logger.debug("Loading ensemble setup from %s/ensemble.json", folder)
-        with open(folder + "/ensemble.json", "r") as f:
+        with open(f"{folder}/ensemble.json", "r") as f:
             settings = json.load(f)
 
         self.n_estimators = int(settings["n_estimators"])
@@ -644,7 +642,7 @@ class Ensemble:
         self.estimators = []
         for i in range(self.n_estimators):
             estimator = self._get_estimator_class(estimator_type)()
-            estimator.load(folder + "/estimator_" + str(i))
+            estimator.load(f"{folder}/estimator_{i}")
             self.estimators.append(estimator)
         self._check_consistency()
 
@@ -658,8 +656,8 @@ class Ensemble:
         ------
         RuntimeError
             Estimators are inconsistent.
-
         """
+
         # Accumulate methods of all estimators
         all_types = [self._get_estimator_type(estimator) for estimator in self.estimators]
         all_n_parameters = [estimator.n_parameters for estimator in self.estimators]
@@ -674,12 +672,12 @@ class Ensemble:
 
             if self.estimator_type != estimator_type:
                 raise RuntimeError(
-                    "Ensemble with inconsistent estimator methods! All methods have to be either"
-                    " single-parameterized ratio estimators, doubly parameterized ratio estimators,"
-                    " or local score estimators. Found types " + ", ".join(all_types) + "."
+                    f"Ensemble with inconsistent estimator methods! All methods have to be either "
+                    f"single-parameterized ratio estimators, doubly parameterized ratio estimators, "
+                    f"or local score estimators. Found types {', '.join(all_types)}."
                 )
 
-        # Check consistency of parameter and observable numnbers
+        # Check consistency of parameter and observable numbers
         self.n_parameters = None
         self.n_observables = None
 
@@ -691,11 +689,13 @@ class Ensemble:
 
             if self.n_parameters is not None and self.n_parameters != estimator_n_parameters:
                 raise RuntimeError(
-                    "Ensemble with inconsistent numbers of parameters for different estimators: %s", all_n_parameters
+                    f"Ensemble with inconsistent numbers of parameters for different estimators: "
+                    f"{all_n_parameters}"
                 )
             if self.n_observables is not None and self.n_observables != estimator_n_observables:
                 raise RuntimeError(
-                    "Ensemble with inconsistent numbers of parameters for different estimators: %s", all_n_observables
+                    f"Ensemble with inconsistent numbers of observables for different estimators: "
+                    f"{all_n_observables}"
                 )
 
     @staticmethod
@@ -725,4 +725,4 @@ class Ensemble:
         elif estimator_type == "likelihood":
             return LikelihoodEstimator
         else:
-            raise RuntimeError("Unknown estimator type {}!".format(estimator_type))
+            raise RuntimeError(f"Unknown estimator type {estimator_type}!")
