@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import logging
 import matplotlib
 import numpy as np
@@ -92,6 +90,7 @@ def plot_fisher_information_contours_2d(
 
     ax: axes or None, optional
         Predefined axes as part of figure instead of standalone figure. Default: None
+
     Returns
     -------
     figure : Figure
@@ -105,7 +104,8 @@ def plot_fisher_information_contours_2d(
 
     if fisher_information_matrices.shape != (n_matrices, 2, 2):
         raise RuntimeError(
-            "Fisher information matrices have shape {}, not (n, 2,2)!".format(fisher_information_matrices.shape)
+            f"Fisher information matrices have shape {fisher_information_matrices.shape}. "
+            f"Not (n, 2,2)!"
         )
 
     if fisher_information_covariances is None:
@@ -219,7 +219,11 @@ def plot_fisher_information_contours_2d(
 
 
 def plot_fisherinfo_barplot(
-    fisher_information_matrices, labels, determinant_indices=None, eigenvalue_colors=None, bar_colors=None
+    fisher_information_matrices,
+    labels,
+    determinant_indices=None,
+    eigenvalue_colors=None,
+    bar_colors=None,
 ):
     """
 
@@ -301,7 +305,7 @@ def plot_fisherinfo_barplot(
         bar_colors_light = bar_colors
 
     if eigenvalue_colors is None:
-        eigenvalue_colors = ["C{}".format(str(i)) for i in range(10)]
+        eigenvalue_colors = [f"C{i}" for i in range(10)]
     eigenvalue_linewidth = 1.5
 
     # Upper plot
@@ -322,9 +326,7 @@ def plot_fisherinfo_barplot(
 
             fraction_finished = 0.0
 
-            for component in range(len(composition)):
-                fraction = composition[component]
-
+            for component, fraction in enumerate(composition):
                 if fraction >= minimal_fraction_for_plot:
                     plt.hlines(
                         [eigenvalue],
