@@ -2,9 +2,15 @@ import logging
 import numpy as np
 import os
 import uproot3
-from collections import OrderedDict
-from particle import Particle
 
+from collections import OrderedDict
+from typing import Callable
+from typing import Dict
+from typing import List
+
+from particle import Particle
+from madminer.models import Cut
+from madminer.models import Observable
 from madminer.utils.particle import MadMinerParticle
 from madminer.utils.various import math_commands
 
@@ -13,11 +19,8 @@ logger = logging.getLogger(__name__)
 
 def parse_delphes_root_file(
     delphes_sample_file,
-    observables,
-    observables_required,
-    observables_defaults,
-    cuts,
-    cuts_default_pass,
+    observables: Dict[str, Observable],
+    cuts: List[Cut],
     weight_labels=None,
     use_generator_truth=False,
     acceptance_pt_min_e=None,
@@ -122,46 +125,41 @@ def parse_delphes_root_file(
     # Observations
     observable_values = OrderedDict()
 
-    for obs_name, obs_definition in observables.items():
+    for name, observable in observables.items():
         values_this_observable = []
+        definition = observable.val_expression
+        default = observable.val_default
 
         # Loop over events
         for event in range(n_events):
             variables = get_objects(event)
 
-            if isinstance(obs_definition, str):
-                try:
-                    values_this_observable.append(eval(obs_definition, variables))
-                except (SyntaxError, NameError, TypeError, ZeroDivisionError, IndexError):
-                    default = observables_defaults[obs_name]
-                    if default is None:
-                        default = np.nan
-                    values_this_observable.append(default)
-            else:
-                try:
-                    values_this_observable.append(
-                        obs_definition(
-                            leptons_all_events[event],
-                            photons_all_events[event],
-                            jets_all_events[event],
-                            met_all_events[event][0],
-                        )
+            try:
+                if isinstance(definition, str):
+                    value = eval(definition, variables)
+                elif isinstance(definition, Callable):
+                    value = definition(
+                        leptons_all_events[event],
+                        photons_all_events[event],
+                        jets_all_events[event],
+                        met_all_events[event][0],
                     )
-                except RuntimeError:
-                    default = observables_defaults[obs_name]
-                    if default is None:
-                        default = np.nan
-                    values_this_observable.append(default)
+                else:
+                    raise TypeError("Not a valid observable")
+            except (IndexError, NameError, RuntimeError, SyntaxError, TypeError, ZeroDivisionError):
+                value = default if default is not None else np.nan
+            finally:
+                values_this_observable.append(value)
 
-        values_this_observable = np.array(values_this_observable, dtype=np.float)
-        observable_values[obs_name] = values_this_observable
+        values_this_observable = np.array(values_this_observable, dtype=np.float64)
+        observable_values[name] = values_this_observable
 
-        logger.debug("  First 10 values for observable %s:\n%s", obs_name, values_this_observable[:10])
+        logger.debug("  First 10 values for observable %s:\n%s", name, values_this_observable[:10])
 
     # Cuts
     cut_values = []
 
-    for cut, default_pass in zip(cuts, cuts_default_pass):
+    for cut in cuts:
         values_this_cut = []
 
         # Loop over events
@@ -172,28 +170,32 @@ def parse_delphes_root_file(
                 variables[obs_name] = observable_values[obs_name][event]
 
             try:
-                values_this_cut.append(eval(cut, variables))
+                value = eval(cut.val_expression, variables)
             except (SyntaxError, NameError, TypeError, ZeroDivisionError, IndexError):
-                values_this_cut.append(default_pass)
+                value = cut.is_required
+            finally:
+                values_this_cut.append(value)
 
-        values_this_cut = np.array(values_this_cut, dtype=np.bool)
+        values_this_cut = np.array(values_this_cut, dtype=bool)
         cut_values.append(values_this_cut)
 
     # Check for existence of required observables
     combined_filter = None
 
-    for obs_name, obs_required in observables_required.items():
-        if obs_required:
-            this_filter = np.isfinite(observable_values[obs_name])
-            n_pass = np.sum(this_filter)
-            n_fail = np.sum(np.invert(this_filter))
+    for name, observable in observables.items():
+        if not observable.is_required:
+            continue
 
-            logger.debug("  %s / %s events pass required observable %s", n_pass, n_pass + n_fail, obs_name)
+        this_filter = np.isfinite(observable_values[name])
+        n_pass = np.sum(this_filter)
+        n_fail = np.sum(np.invert(this_filter))
 
-            if combined_filter is None:
-                combined_filter = this_filter
-            else:
-                combined_filter = np.logical_and(combined_filter, this_filter)
+        logger.debug("  %s / %s events pass required observable %s", n_pass, n_pass + n_fail, name)
+
+        if combined_filter is None:
+            combined_filter = this_filter
+        else:
+            combined_filter = np.logical_and(combined_filter, this_filter)
 
     # Check cuts
     for cut, values_this_cut in zip(cuts, cut_values):
@@ -213,7 +215,7 @@ def parse_delphes_root_file(
         n_fail = np.sum(np.invert(combined_filter))
 
         if n_pass == 0:
-            logger.warning("  No observations remainining!")
+            logger.warning("  No observations remaining!")
 
             return None, None, combined_filter
 
@@ -332,7 +334,7 @@ def _get_particles_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
         event_masses = np.concatenate((0.105 * np.ones_like(pt_mu[ievent]), 0.000511 * np.ones_like(pt_e[ievent])))
         event_charges = np.concatenate((charge_mu[ievent], charge_e[ievent]))
         event_pdgid_positive_charges = np.concatenate(
-            (-13 * np.ones_like(pt_mu[ievent], dtype=np.int), -11 * np.ones_like(pt_e[ievent], dtype=np.int))
+            (-13 * np.ones_like(pt_mu[ievent], dtype=int), -11 * np.ones_like(pt_e[ievent], dtype=int))
         )
 
         # Sort by descending pT
