@@ -1,8 +1,6 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
+import logging
 import os
 import shutil
-import logging
 
 from madminer.utils.various import call_command, make_file_executable, create_missing_folders
 
@@ -17,7 +15,6 @@ def generate_mg_process(
     ufo_model_directory=None,
     log_file=None,
     initial_command=None,
-    explicit_python_call=False,
     python_executable=None,
 ):
 
@@ -49,9 +46,6 @@ def generate_mg_process(
     log_file : str or None, optional
         Path to a log file in which the MadGraph output is saved. Default value: None.
 
-    explicit_python_call : bool, optional
-        Calls `python2.7` instead of `python`.
-
     python_executable : None or str, optional
         Overwrites the default Python executable
 
@@ -70,31 +64,22 @@ def generate_mg_process(
         copy_ufo_model(ufo_model_directory, mg_directory)
 
     # MG commands
-    temp_proc_card_file = temp_directory + "/generate.mg5"
+    temp_proc_card_file = f"{temp_directory}/generate.mg5"
     shutil.copyfile(proc_card_file, temp_proc_card_file)
 
     with open(temp_proc_card_file, "a") as myfile:
-        myfile.write("\n\noutput " + mg_process_directory)
+        myfile.write(f"\n")
+        myfile.write(f"\n")
+        myfile.write(f"output {mg_process_directory}")
 
-    # Call MG5
-    if initial_command is None:
-        initial_command = ""
-    else:
-        initial_command = initial_command + "; "
+    # Call specific initial command and Python binary
+    initial_command = f"{initial_command}; " if initial_command is not None else ""
+    python_binary = f"{python_executable} " if python_executable is not None else ""
 
-    # Explicitly call Python 2 if necessary
-    if explicit_python_call:
-        python_call = python_executable + " " if python_executable is not None else "python2.7 "
-    else:
-        python_call = ""
+    command = f"{initial_command}{python_binary}{mg_directory}/bin/mg5_aMC {temp_proc_card_file}"
+    logger.info(f"Calling MadGraph: {command}")
 
-    logger.info(
-        "Calling MadGraph: %s", initial_command + python_call + mg_directory + "/bin/mg5_aMC " + temp_proc_card_file
-    )
-
-    _ = call_command(
-        initial_command + python_call + mg_directory + "/bin/mg5_aMC " + temp_proc_card_file, log_file=log_file
-    )
+    _ = call_command(cmd=command, log_file=log_file)
 
 
 def setup_mg_with_scripts(
@@ -110,7 +95,6 @@ def setup_mg_with_scripts(
     initial_command=None,
     log_dir=None,
     log_file_from_logdir=None,
-    explicit_python_call=False,
     order="LO",
     python_executable=None,
 ):
@@ -166,9 +150,6 @@ def setup_mg_with_scripts(
         Path to a log file in which the MadGraph output is saved, relative from the default log directory. Default
         value: None.
 
-    explicit_python_call : bool, optional
-        Calls `python2.7` instead of `python`.
-
     python_executable : None or str, optional
         Overwrites the default Python executable
 
@@ -198,19 +179,19 @@ def setup_mg_with_scripts(
     # Find filenames for process card and script
     if proc_card_filename_from_mgprocdir is None:
         for i in range(1000):
-            proc_card_filename_from_mgprocdir = "/Cards/start_event_generation_{}.mg5".format(i)
-            if not os.path.isfile(mg_process_directory + "/" + proc_card_filename_from_mgprocdir):
+            proc_card_filename_from_mgprocdir = f"/Cards/start_event_generation_{i}.mg5"
+            if not os.path.isfile(f"{mg_process_directory}/{proc_card_filename_from_mgprocdir}"):
                 break
     else:
         proc_card_filename = mg_process_directory + "/" + proc_card_filename_from_mgprocdir
 
     if script_file_from_mgprocdir is None:
         for i in range(1000):
-            script_file = mg_process_directory + "/madminer/scripts/madminer_run_{}.sh".format(i)
+            script_file = f"{mg_process_directory}/madminer/scripts/madminer_run_{i}.sh"
             if not os.path.isfile(script_file):
                 break
     else:
-        script_file = mg_process_directory + "/" + script_file_from_mgprocdir
+        script_file = f"{mg_process_directory}/{script_file_from_mgprocdir}"
 
     script_filename = os.path.basename(script_file)
 
@@ -236,54 +217,43 @@ def setup_mg_with_scripts(
     with open(proc_card_filename, "w") as file:
         file.write(mg_commands)
 
-    # Initial commands
-    if initial_command is None:
-        initial_command = ""
-
     #  Card copying commands
     copy_commands = ""
     if run_card_file_from_mgprocdir is not None:
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            run_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/run_card.dat",
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{run_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/run_card.dat\n"
         )
     if param_card_file_from_mgprocdir is not None:
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            param_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/param_card.dat",
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{param_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/param_card.dat\n"
         )
     if reweight_card_file_from_mgprocdir is not None and not is_background:
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            reweight_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/reweight_card.dat",
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{reweight_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/reweight_card.dat\n"
         )
     if pythia8_card_file_from_mgprocdir is not None and order == "LO":
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            pythia8_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/pythia8_card.dat",
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{pythia8_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/pythia8_card.dat\n"
         )
-    elif pythia8_card_file_from_mgprocdir is not None and order == "NLO":
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            pythia8_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/shower_card.dat",
+    if pythia8_card_file_from_mgprocdir is not None and order == "NLO":
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{pythia8_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/shower_card.dat\n"
         )
-
     if configuration_file_from_mgprocdir is not None:
-        copy_commands += "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            configuration_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/me5_configuration.txt",
+        copy_commands += (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{configuration_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/me5_configuration.txt\n"
         )
 
     # Replace environment variable in proc card
@@ -294,11 +264,9 @@ def setup_mg_with_scripts(
         "Cards/mg_commands.mg5",
     )
 
-    # Explicitly call Python 2 if necessary
-    if explicit_python_call:
-        python_call = python_executable + " " if python_executable is not None else "python2.7 "
-    else:
-        python_call = ""
+    # Call specific initial command and Python binary
+    initial_command = f"{initial_command} " if initial_command is not None else ""
+    python_binary = f"{python_executable} " if python_executable is not None else ""
 
     # Put together script
     script = (
@@ -310,7 +278,7 @@ def setup_mg_with_scripts(
         placeholder_definition,
         copy_commands,
         replacement_command,
-        python_call,
+        python_binary,
         mg_directory_placeholder,
         mg_process_directory_placeholder,
         "Cards/mg_commands.mg5",
@@ -323,12 +291,11 @@ def setup_mg_with_scripts(
     make_file_executable(script_file)
 
     # How to call it from master script
-    call_placeholder = "{}/{} {} {} {}".format(
-        mg_process_directory_placeholder,
-        script_file_from_mgprocdir,
-        mg_directory_placeholder,
-        mg_process_directory_placeholder,
-        log_dir_placeholder,
+    call_placeholder = (
+        f"{mg_process_directory_placeholder}/{script_file_from_mgprocdir} "
+        f"{mg_directory_placeholder} "
+        f"{mg_process_directory_placeholder} "
+        f"{log_dir_placeholder}"
     )
 
     return call_placeholder
@@ -346,7 +313,6 @@ def run_mg(
     is_background=False,
     initial_command=None,
     log_file=None,
-    explicit_python_call=False,
     order="LO",
     python_executable=None,
 ):
@@ -396,8 +362,8 @@ def run_mg(
     log_file : str or None, optional
         Path to a log file in which the MadGraph output is saved. Default value: None.
 
-    explicit_python_call : bool, optional
-        Calls `python2.7` instead of `python`.
+    python_executable : None or str, optional
+        Overwrites the default Python executable
 
     Returns
     -------
@@ -415,22 +381,22 @@ def run_mg(
 
     # Copy cards
     if run_card_file is not None:
-        shutil.copyfile(run_card_file, mg_process_directory + "/Cards/run_card.dat")
+        shutil.copyfile(run_card_file, f"{mg_process_directory}/Cards/run_card.dat")
     if param_card_file is not None:
-        shutil.copyfile(param_card_file, mg_process_directory + "/Cards/param_card.dat")
+        shutil.copyfile(param_card_file, f"{mg_process_directory}/Cards/param_card.dat")
     if reweight_card_file is not None and not is_background:
-        shutil.copyfile(reweight_card_file, mg_process_directory + "/Cards/reweight_card.dat")
+        shutil.copyfile(reweight_card_file, f"{mg_process_directory}/Cards/reweight_card.dat")
     if pythia8_card_file is not None and order == "LO":
-        shutil.copyfile(pythia8_card_file, mg_process_directory + "/Cards/pythia8_card.dat")
+        shutil.copyfile(pythia8_card_file, f"{mg_process_directory}/Cards/pythia8_card.dat")
     if pythia8_card_file is not None and order == "NLO":
-        shutil.copyfile(pythia8_card_file, mg_process_directory + "/Cards/shower_card.dat")
+        shutil.copyfile(pythia8_card_file, f"{mg_process_directory}/Cards/shower_card.dat")
     if configuration_card_file is not None:
-        shutil.copyfile(configuration_card_file, mg_process_directory + "/Cards/me5_configuration.txt")
+        shutil.copyfile(configuration_card_file, f"{mg_process_directory}/Cards/me5_configuration.txt")
 
     # Find filenames for process card and script
     if proc_card_filename is None:
         for i in range(1000):
-            proc_card_filename = mg_process_directory + "/Cards/start_event_generation_{}.mg5".format(i)
+            proc_card_filename = f"{mg_process_directory}/Cards/start_event_generation_{i}.mg5"
             if not os.path.isfile(proc_card_filename):
                 break
 
@@ -453,21 +419,14 @@ def run_mg(
     with open(proc_card_filename, "w") as file:
         file.write(mg_commands)
 
-    # Call MG5
-    if initial_command is None:
-        initial_command = ""
-    else:
-        initial_command = initial_command + "; "
+    # Call specific initial command and Python binary
+    initial_command = f"{initial_command}; " if initial_command is not None else ""
+    python_binary = f"{python_executable} " if python_executable is not None else ""
 
-    # Explicitly call Python 2 if necessary
-    if explicit_python_call:
-        python_call = python_executable + " " if python_executable is not None else "python2.7 "
-    else:
-        python_call = ""
+    command = f"{initial_command}{python_binary}{mg_directory}/bin/mg5_aMC {proc_card_filename}"
+    logger.info(f"Calling MadGraph: {command}")
 
-    _ = call_command(
-        initial_command + python_call + mg_directory + "/bin/mg5_aMC " + proc_card_filename, log_file=log_file
-    )
+    _ = call_command(cmd=command, log_file=log_file)
 
 
 def setup_mg_reweighting_with_scripts(
@@ -525,10 +484,11 @@ def setup_mg_reweighting_with_scripts(
     log_dir_placeholder = "$mmlogdir"
     placeholder_definition = r"mgprocdir=${1:-" + mg_process_directory + r"}" + "\n"
     placeholder_definition += r"mmlogdir=${2:-" + log_dir + r"}"
+
     if script_file_from_mgprocdir is None:
-        script_file = mg_process_directory + "/madminer/scripts/madminer_reweight_{}.sh".format(run_name)
+        script_file = f"{mg_process_directory}/madminer/scripts/madminer_reweight_{run_name}.sh"
     else:
-        script_file = mg_process_directory + "/" + script_file_from_mgprocdir
+        script_file = f"{mg_process_directory}/{script_file_from_mgprocdir}"
 
     script_filename = os.path.basename(script_file)
 
@@ -541,11 +501,10 @@ def setup_mg_reweighting_with_scripts(
 
     #  Card copying commands
     if reweight_card_file_from_mgprocdir is not None:
-        copy_commands = "cp {}/{} {}{}\n".format(
-            mg_process_directory_placeholder,
-            reweight_card_file_from_mgprocdir,
-            mg_process_directory_placeholder,
-            "/Cards/reweight_card.dat",
+        copy_commands = (
+            f"cp "
+            f"{mg_process_directory_placeholder}/{reweight_card_file_from_mgprocdir} "
+            f"{mg_process_directory_placeholder}/Cards/reweight_card.dat\n"
         )
     else:
         copy_commands = ""
@@ -570,8 +529,9 @@ def setup_mg_reweighting_with_scripts(
     make_file_executable(script_file)
 
     # How to call it from master script
-    call_instruction = "{}/{} [MG_process_directory] [log_directory]".format(
-        mg_process_directory, script_file_from_mgprocdir
+    call_instruction = (
+        f"{mg_process_directory}/{script_file_from_mgprocdir} "
+        f"[MG_process_directory] [log_directory]"
     )
 
     return call_instruction
@@ -618,19 +578,17 @@ def run_mg_reweighting(mg_process_directory, run_name, reweight_card_file=None,
         shutil.copyfile(reweight_card_file, mg_process_directory + "/Cards/reweight_card.dat")
 
     # Call MG5 reweight feature
-    if initial_command is None:
-        initial_command = ""
-    else:
-        initial_command = initial_command + "; "
+    initial_command = f"{initial_command}; " if initial_command else ""
 
     _ = call_command(
-        "{}{}/bin/madevent reweight {} -f".format(initial_command, mg_process_directory, run_name), log_file=log_file
+        cmd=f"{initial_command}{mg_process_directory}/bin/madevent reweight {run_name} -f",
+        log_file=log_file,
     )
 
 
 def copy_ufo_model(ufo_directory, mg_directory):
     _, model_name = os.path.split(ufo_directory)
-    destination = mg_directory + "/models/" + model_name
+    destination = f"{mg_directory}/models/{model_name}"
 
     if os.path.isdir(destination):
         return
@@ -648,6 +606,7 @@ def create_master_script(log_directory, master_script_filename, mg_directory, mg
         + "# Usage: run.sh [MG_directory] [MG_process_directory] [log_directory]\n\n"
         + "{}\n\n{}"
     ).format(placeholder_definition, commands)
+
     with open(master_script_filename, "w") as file:
         file.write(script)
     make_file_executable(master_script_filename)
