@@ -1,16 +1,14 @@
-from __future__ import absolute_import, division, print_function
-
+import logging
 import numpy as np
 import numpy.random as rng
 import torch
-from torch import tensor
 import torch.nn as nn
 import torch.nn.functional as F
 
 from madminer.utils.ml.models.base import BaseFlow, BaseConditionalFlow
 from madminer.utils.ml.models.masks import create_degrees, create_masks, create_weights, create_weights_conditional
 from madminer.utils.ml.utils import get_activation_function
-import logging
+from torch import tensor
 
 logger = logging.getLogger(__name__)
 
@@ -19,7 +17,7 @@ class GaussianMADE(BaseFlow):
     """ """
 
     def __init__(self, n_inputs, n_hiddens, activation="relu", input_order="sequential", mode="sequential"):
-        super(GaussianMADE, self).__init__(n_inputs)
+        super().__init__(n_inputs)
 
         # save input arguments
         self.activation = activation
@@ -52,7 +50,6 @@ class GaussianMADE(BaseFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -94,7 +91,6 @@ class GaussianMADE(BaseFlow):
              (Default value = None)
         **kwargs :
 
-
         Returns
         -------
 
@@ -131,7 +127,6 @@ class GaussianMADE(BaseFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -139,7 +134,7 @@ class GaussianMADE(BaseFlow):
         self.to_args = args
         self.to_kwargs = kwargs
 
-        self = super(GaussianMADE, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, (M, W, b) in enumerate(zip(self.Ms, self.Ws, self.bs)):
             self.Ms[i] = M.to(*args, **kwargs)
@@ -159,9 +154,15 @@ class ConditionalGaussianMADE(BaseConditionalFlow):
     """ """
 
     def __init__(
-        self, n_conditionals, n_inputs, n_hiddens, activation="relu", input_order="sequential", mode="sequential"
+        self,
+        n_conditionals,
+        n_inputs,
+        n_hiddens,
+        activation="relu",
+        input_order="sequential",
+        mode="sequential",
     ):
-        super(ConditionalGaussianMADE, self).__init__(n_conditionals, n_inputs)
+        super().__init__(n_conditionals, n_inputs)
 
         # save input arguments
         self.activation = activation
@@ -199,7 +200,6 @@ class ConditionalGaussianMADE(BaseConditionalFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -267,7 +267,6 @@ class ConditionalGaussianMADE(BaseConditionalFlow):
              (Default value = None)
         **kwargs :
 
-
         Returns
         -------
 
@@ -306,7 +305,6 @@ class ConditionalGaussianMADE(BaseConditionalFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -314,7 +312,7 @@ class ConditionalGaussianMADE(BaseConditionalFlow):
         self.to_args = args
         self.to_kwargs = kwargs
 
-        self = super(ConditionalGaussianMADE, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, (M, W, b) in enumerate(zip(self.Ms, self.Ws, self.bs)):
             self.Ms[i] = M.to(*args, **kwargs)
